from abc import ABC, abstractmethod


class BaseLogger(ABC):

    @abstractmethod
    def log(self, source: str, message: str, data: object = None):
        pass


class Logger(BaseLogger):
    def __init__(self):
        pass

    def log(self, source: str, message: str, data: object = None):
        print(f'source:{source}, message:{message}, data:{data}')


class LoggerFactory:
    @classmethod
    def get_logger(cls):
        if not hasattr(cls, "logger") or cls.logger is None:
            cls.logger = Logger()
        return cls.logger

    @classmethod
    def set_logger(cls, logger):
        cls.logger = logger



from pydiator_core.serializer import BaseSerializer, SerializerFactory, Serializer
from tests.base_test_case import BaseTestCase, TestResponse


class TestSerializerFactory(BaseTestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_set_serializer(self):
        # Given
        class TestSerializerObj(BaseSerializer):

            def dumps(self, obj):
                return ""

            def loads(self, obj):
                return "test"

            def deserialize(self, obj):
                return self.loads(self.dumps(obj))

        test_serializer = TestSerializerObj()
        SerializerFactory.set_serializer(serializer=test_serializer)

        test_response = TestResponse(True)

        # When
        response = test_response.to_json()

        # Then
        assert response == "test"
        assert test_serializer == SerializerFactory.get_serializer()

    def test_serializer_when_does_not_call_set(self):
        # Given

        # When
        current_serializer = SerializerFactory.get_serializer()

        # Then
        assert isinstance(current_serializer, Serializer)



from unittest import mock
from unittest.mock import MagicMock

from pydiator_core.interfaces import BaseNotification
from pydiator_core.logger import LoggerFactory, BaseLogger
from pydiator_core.mediatr import Mediatr
from pydiator_core.serializer import SerializerFactory, BaseSerializer
from tests.base_test_case import BaseTestCase, TestRequest, TestResponse, FakeMediatrContainer, TestNotification


class TestMediatrContainer(BaseTestCase):

    def setUp(self):
        SerializerFactory.set_serializer(None)
        LoggerFactory.set_logger(None)

    def tearDown(self):
        pass

    def test_read_default_values_when_create_instance(self):
        # Given

        # When
        mediatr = Mediatr()

        # Then
        assert mediatr.is_ready is False

    def test_ready_when_is_ready(self):
        # Given
        mediatr = Mediatr()
        mediatr.is_ready = True

        # When
        mediatr.ready(container=None)

        # Then
        assert mediatr.is_ready

    def test_ready_when_is_not_ready(self):
        # Given
        container = FakeMediatrContainer()
        mediatr = Mediatr()

        # When
        mediatr.ready(container=container)

        # Then
        assert mediatr.is_ready
        assert len(container.get_pipelines()) == 1

    def test_ready_when_container_is_none(self):
        # Given
        mediatr = Mediatr()

        # When
        with self.assertRaises(Exception) as context:
            mediatr.ready(container=None)

        # Then
        assert mediatr.is_ready is False
        assert 'mediatr_container_is_none' == context.exception.args[0]

    def test_ready_when_serializer_is_not_none(self):
        # Given
        container = FakeMediatrContainer()
        mediatr = Mediatr()

        # When
        mediatr.ready(container=container, serializer={})

        # Then
        assert mediatr.is_ready is True
        assert SerializerFactory.get_serializer() == {}

    def test_ready_when_serializer_is_none(self):
        # Given
        mediatr_container = FakeMediatrContainer()
        mediatr = Mediatr()

        # When
        mediatr.ready(container=mediatr_container)

        # Then
        assert mediatr.is_ready is True
        assert isinstance(SerializerFactory.get_serializer(), BaseSerializer)

    def test_ready_when_container_and_serializer_set(self):
        # Given
        container = FakeMediatrContainer()
        mediatr = Mediatr()

        # When
        mediatr.ready(container=container, serializer={})

        # Then
        assert mediatr.is_ready is True
        assert not isinstance(SerializerFactory.get_serializer(), BaseSerializer)
        assert SerializerFactory.get_serializer() == {}
        assert len(container.get_pipelines()) == 1

    def test_ready_when_container_and_logger_set(self):
        # Given
        container = FakeMediatrContainer()
        mediatr = Mediatr()

        # When
        mediatr.ready(container=container, logger={})

        # Then
        assert mediatr.is_ready is True
        assert not isinstance(LoggerFactory.get_logger(), BaseLogger)
        assert LoggerFactory.get_logger() == {}
        assert len(container.get_pipelines()) == 1

    def test_ready_when_logger_is_none(self):
        # Given
        container = FakeMediatrContainer()
        mediatr = Mediatr()

        # When
        mediatr.ready(container=container)

        # Then
        assert mediatr.is_ready is True
        assert isinstance(LoggerFactory.get_logger(), BaseLogger)

    def test_send_raise_exception_when_container_is_none(self):
        # Given
        mediatr = Mediatr()

        # When
        with self.assertRaises(Exception) as context:
            self.async_loop(mediatr.send(TestRequest()))

        # Then
        assert 'mediatr_container_is_none' == context.exception.args[0]

    def test_send_raise_exception_when_container_pipelines_is_empty(self):
        # Given
        container = MagicMock()
        container.get_pipelines.return_value = []
        mediatr = Mediatr()
        mediatr.ready(container=container)

        # When
        with self.assertRaises(Exception) as context:
            self.async_loop(mediatr.send(TestRequest()))

        # Then
        assert 'mediatr_container_has_not_contain_any_pipeline' == context.exception.args[0]

    @mock.patch("pydiator_core.mediatr.DefaultPipeline")
    def test_send_return_default_pipeline_result_when_container_pipelines_is_empty(self, mock_default_pipeline):
        # Given
        next_response = TestResponse(success=True)

        async def next_handle(req):
            return next_response

        mock_default_pipeline.return_value.handle = next_handle
        container = FakeMediatrContainer()
        mediatr = Mediatr()
        mediatr.ready(container)

        # When
        response = self.async_loop(mediatr.send(TestRequest()))

        # Then
        assert response is next_response
        assert response.success

    def test_publish_when_handlers_is_empty(self):
        # Given
        container = FakeMediatrContainer()
        mediatr = Mediatr()
        container.register_notification(TestNotification(), [])
        mediatr.ready(container)

        # When
        with self.assertRaises(Exception) as context:
            self.async_loop(mediatr.publish(BaseNotification()))

        # Then
        assert 'mediatr_container_has_not_contain_any_notification_handler_for:BaseNotification' == \
               context.exception.args[0]

    def test_publish_when_handlers_exist(self):
        # Given
        async def next_handle(notification):
            pass

        mock_notification_handler = mock.MagicMock()
        mock_notification_handler.handle.side_effect = next_handle

        container = FakeMediatrContainer()
        container.register_notification(TestNotification(), [mock_notification_handler])

        mediatr = Mediatr()
        mediatr.ready(container)

        # When
        self.async_loop(mediatr.publish(TestNotification()))

        # Then
        assert mock_notification_handler.handle.called
        assert mock_notification_handler.handle.call_count == 1

    def test_publish_when_handle_has_exception(self):
        # Given
        async def next_handle(notification):
            raise Exception("")

        mock_notification_handler = mock.MagicMock()
        mock_notification_handler.handle.side_effect = next_handle

        container = FakeMediatrContainer()
        container.register_notification(TestNotification(), [mock_notification_handler])

        mediatr = Mediatr()
        mediatr.ready(container)

        # When
        self.async_loop(mediatr.publish(notification=TestNotification()))

        # Then
        assert mock_notification_handler.handle.called
        assert mock_notification_handler.handle.call_count == 1

    def test_publish_when_handle_has_exception_and_throw_exception(self):
        # Given
        async def next_handle(notification):
            raise Exception("test_exception")

        mock_notification_handler = mock.MagicMock()
        mock_notification_handler.handle.side_effect = next_handle

        container = FakeMediatrContainer()
        container.register_notification(TestNotification(), [mock_notification_handler])

        mediatr = Mediatr()
        mediatr.ready(container)

        # When
        with self.assertRaises(Exception) as context:
            self.async_loop(mediatr.publish(notification=TestNotification(), throw_exception=True))

        # Then
        assert mock_notification_handler.handle.called
        assert mock_notification_handler.handle.call_count == 1
        assert 'test_exception' == context.exception.args[0]



from pydiator_core.interfaces import BaseRequest, BasePipeline, BaseCacheable, CacheType, BaseCacheProvider
from pydiator_core.serializer import SerializerFactory


class CachePipeline(BasePipeline):
    def __init__(self, cache_provider: BaseCacheProvider) -> None:
        self.cache_provider = cache_provider

    async def handle(self, req: BaseRequest, **kwargs) -> object:
        if self.next() is None:
            raise Exception("pydiator_cache_pipeline_has_no_next_pipeline")

        next_handle = getattr(self.next(), "handle", None)
        if next_handle is None or not callable(next_handle):
            raise Exception("handle_function_of_next_pipeline_is_not_valid_for_cache_pipeline")

        if self.cache_provider is None:
            return await next_handle(req=req, **kwargs)

        if isinstance(req, BaseCacheable):
            cache_type = req.get_cache_type()
            if cache_type != CacheType.NONE and req.is_no_cache() is False:
                cache_key = req.get_cache_key()
                if cache_key is not None and cache_key != "":
                    if cache_type == CacheType.DISTRIBUTED:
                        cached_obj = self.__get_from_cache(cache_key)
                        if cached_obj is not None:
                            return cached_obj

                        response = await next_handle(req, **kwargs)

                        cache_duration = req.get_cache_duration()
                        if response is not None and response != "" and cache_duration > 0:
                            self.__add_to_cache(response, cache_key, cache_duration)

                        return response

        return await next_handle(req, **kwargs)

    def __get_from_cache(self, cache_key) -> object:
        cached_obj_str = self.cache_provider.get(cache_key)
        if cached_obj_str is not None:
            return self.__get_serializer().loads(cached_obj_str)

        return None

    def __add_to_cache(self, res: object, cache_key, cache_duration):
        res_value_obj = self.__get_serializer().dumps(res)
        if res_value_obj is not None:
            self.cache_provider.add(cache_key, res_value_obj, cache_duration)

    @staticmethod
    def __get_serializer():
        return SerializerFactory.get_serializer()



from pydiator_core.interfaces import BaseRequest, BaseNotification, BaseMediatr, BaseMediatrContainer
from pydiator_core.default_pipeline import DefaultPipeline
from pydiator_core.logger import LoggerFactory, BaseLogger
from pydiator_core.serializer import BaseSerializer, SerializerFactory


class Mediatr(BaseMediatr):

    def __init__(self):
        self.__container = None
        self.is_ready = False

    def ready(self, container: BaseMediatrContainer, serializer: BaseSerializer = None, logger: BaseLogger = None):
        if self.is_ready:
            return

        if container is None:
            raise Exception("mediatr_container_is_none")

        self.__container = container
        self.__container.prepare_pipes(DefaultPipeline(self.__container))

        if serializer is not None:
            SerializerFactory.set_serializer(serializer)

        if logger is not None:
            LoggerFactory.set_logger(logger)

        self.is_ready = True

    async def send(self, req: BaseRequest, **kwargs) -> object:
        if self.__container is None:
            raise Exception("mediatr_container_is_none")

        pipelines = self.__container.get_pipelines()
        if len(pipelines) == 0:
            raise Exception("mediatr_container_has_not_contain_any_pipeline")

        return await pipelines[0].handle(req=req, **kwargs)

    async def publish(self, notification: BaseNotification, throw_exception: bool = False):
        notification_type_name = notification.get_class_name()
        notifications_obj = self.__container.get_notifications()
        if notification_type_name not in notifications_obj:
            raise Exception(f"mediatr_container_has_not_contain_any_notification_handler_for:{notification_type_name}")

        handlers = notifications_obj[notification_type_name]
        for h in handlers:
            if not throw_exception:
                try:
                    await h.handle(notification)
                except Exception as e:
                    print("exception_when_notification_handle:", str(e))
            else:
                await h.handle(notification)


pydiator = Mediatr()



import asyncio
from typing import List
from unittest import TestCase

from pydiator_core.interfaces import BaseRequest, BaseResponse, BaseHandler, BasePipeline, BaseNotification, \
    BaseNotificationHandler, BaseCacheable, CacheType, BaseCacheProvider
from pydiator_core.mediatr_container import MediatrContainer, BaseMediatrContainer
from pydiator_core.mediatr import pydiator


class BaseTestCase(TestCase):
    @staticmethod
    def async_return(result):
        f = asyncio.Future()
        f.set_result(result)
        return f

    @staticmethod
    def async_loop(func):
        loop = asyncio.new_event_loop()
        response = loop.run_until_complete(func)
        loop.close()
        return response

    @staticmethod
    def register_request(req, handler):
        container = MediatrContainer()
        container.register_request(req, handler)
        pydiator.ready(container)


class TestRequest(BaseRequest):
    def __init__(self):
        return


class TestRequestWithCacheable(BaseRequest, BaseCacheable):
    def __init__(self, cache_key, cache_duration, cache_type: CacheType):
        self.cache_key = cache_key
        self.cache_duration = cache_duration
        self.cache_type = cache_type

    def get_cache_key(self) -> str:
        return self.cache_key

    def get_cache_duration(self) -> int:
        return self.cache_duration

    def get_cache_type(self) -> CacheType:
        return self.cache_type


class TestResponse(BaseResponse):
    def __init__(self, success: bool):
        self.success = success


class TestHandler(BaseHandler):
    async def handle(self, req: BaseRequest):
        return TestResponse(success=True)


class TestSyncHandler(BaseHandler):
    def handle(self, req: BaseRequest):
        return TestResponse(success=True)


class TestPipeline(BasePipeline):
    def __init__(self, response_success):
        self.response_success = response_success

    async def handle(self, req: BaseRequest) -> object:
        return TestResponse(success=self.response_success)


class TestNotification(BaseNotification):
    def __init__(self):
        return


class TestNotificationHandler(BaseNotificationHandler):

    async def handle(self, notification: BaseNotification):
        return


class FakeMediatrContainer(BaseMediatrContainer):

    def prepare_pipes(self, pipeline: BasePipeline):
        self.__pipelines.append(pipeline)

    def __init__(self):
        self.__requests = {}
        self.__notifications = {}
        self.__pipelines = []

    def register_request(self, req: BaseRequest, handler: BaseHandler):
        return

    def register_pipeline(self, pipeline: BasePipeline):
        self.__pipelines.append(pipeline)

    def register_notification(self, notification: BaseNotification, handlers: List[BaseNotificationHandler]):
        self.__notifications[type(notification).__name__] = handlers

    def get_requests(self):
        return self.__requests

    def get_notifications(self):
        return self.__notifications

    def get_pipelines(self):
        return self.__pipelines


class FakeCacheProvider(BaseCacheProvider):
    def add(self, key: str, value, expires):
        pass

    def get(self, key: str):
        pass

    def exist(self, key: str):
        pass

    def delete(self, key: str):
        pass

    def check_connection(self):
        pass



import json
import datetime
from abc import ABC, abstractmethod
from decimal import Decimal
from uuid import UUID


class BaseSerializer(ABC):

    @abstractmethod
    def dumps(self, obj):
        pass

    @abstractmethod
    def loads(self, obj):
        pass

    @abstractmethod
    def deserialize(self, obj):
        pass


class SerializerFactory:
    @classmethod
    def get_serializer(cls):
        if not hasattr(cls, "serializer") or cls.serializer is None:
            cls.serializer = Serializer()
        return cls.serializer

    @classmethod
    def set_serializer(cls, serializer):
        cls.serializer = serializer


class Serializer(BaseSerializer):

    def dumps(self, obj):
        """Converts List to Json String"""
        if isinstance(obj, list):
            return json.dumps(obj, default=lambda x: x.__dict__, cls=Serializer.CustomEncoder, sort_keys=True,
                              indent=4)

        """Converts to Json String"""
        if isinstance(obj, dict):
            return json.dumps(obj, cls=Serializer.CustomEncoder, sort_keys=True, indent=4)

        return json.dumps(obj.__dict__, cls=Serializer.CustomEncoder, sort_keys=True, indent=4).encode()

    def loads(self, obj):
        return json.loads(obj)

    def deserialize(self, obj):
        return self.loads(self.dumps(obj=obj))

    class CustomEncoder(json.JSONEncoder):
        def default(self, obj):
            if isinstance(obj, Decimal):
                return float("{:.2f}".format(obj))

            if isinstance(obj, UUID):
                return str(obj)

            if isinstance(obj, datetime.datetime) or isinstance(obj, datetime.date):
                return obj.isoformat()

            if isinstance(obj, object):
                return obj.__dict__

            return json.JSONEncoder.default(self, obj)


class JsonSerializable(object):
    def to_json(self):
        return SerializerFactory.get_serializer().deserialize(obj=self)



from typing import List
from pydiator_core.interfaces import BaseRequest, BasePipeline
from pydiator_core.logger import LoggerFactory
from pydiator_core.serializer import SerializerFactory


class LogPipeline(BasePipeline):
    def __init__(self):
        self.serializer = None
        self.logger = LoggerFactory.get_logger()

    async def handle(self, req: BaseRequest, **kwargs) -> object:
        self.serializer = SerializerFactory.get_serializer()

        if self.next() is None:
            raise Exception("pydiator_log_pipeline_has_no_next_pipeline")

        next_handle = getattr(self.next(), "handle", None)
        if next_handle is None or not callable(next_handle):
            raise Exception("handle_function_of_next_pipeline_is_not_valid_for_log_pipeline")

        response = await next_handle(req=req, **kwargs)

        if hasattr(response, "__dict__") or isinstance(response, List):
            _response = self.serializer.deserialize(response)
        else:
            _response = str(response)

        log_obj = {
            "req": self.serializer.deserialize(req),
            "res": _response
        }

        req_type_name = req.get_class_name()
        self.logger.log(source=self.__class__.__name__, message=req_type_name, data=log_obj)

        return response



import datetime
import uuid
from decimal import Decimal
from typing import List
from uuid import UUID

from pydiator_core.interfaces import BaseResponse
from pydiator_core.serializer import Serializer
from tests.base_test_case import BaseTestCase, TestResponse


class TestSerializer(BaseTestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_dumps_when_type_is_list(self):
        # Given
        serializer = Serializer()
        obj = [{"name": "1"}, {"name": "2"}]

        # When
        response = serializer.dumps(obj)

        # Then
        assert response is not None
        assert len(serializer.loads(response)) == 2

    def test_dumps_when_type_is_dict(self):
        # Given
        serializer = Serializer()
        obj = {
            "name": 1
        }

        # When
        response = serializer.dumps(obj)

        # Then
        assert response is not None
        assert len(serializer.loads(response)) == 1

    def test_dumps(self):
        # Given
        serializer = Serializer()
        obj = TestResponse(success=True)

        # When
        response = serializer.dumps(obj)

        # Then
        assert response is not None
        assert len(serializer.loads(response)) == 1

    def test_loads(self):
        # Given
        serializer = Serializer()
        obj = TestResponse(success=True)
        dump = serializer.dumps(obj)

        # When
        response = serializer.loads(dump)

        # Then
        assert response is not None
        assert response["success"]

    def test_deserialize(self):
        # Given
        class TestItem(BaseResponse):
            def __init__(self, id: int, title: str):
                self.id = id,
                self.title = title

        class TestMixResponse(BaseResponse):
            def __init__(self, text: str, success: bool, dec: Decimal, uid: UUID, dt: datetime.datetime,
                         date: datetime.date,
                         item: TestItem, items: List[TestItem]):
                self.text = text
                self.success = success
                self.dec = dec
                self.uid = uid
                self.dt = dt
                self.date = date
                self.item = item
                self.items = items

        time = datetime.datetime.now()
        date = datetime.datetime.date(time)
        uid = uuid.uuid4()
        item = TestItem(3, "title3")
        items = [TestItem(1, "title1"), TestItem(2, "title2")]
        mix_response = TestMixResponse(text="bla bla",
                                       success=True,
                                       dec=Decimal.from_float(1.123),
                                       uid=uid,
                                       dt=time,
                                       date=date,
                                       item=item,
                                       items=items)

        # When
        response = mix_response.to_json()

        # Then
        assert response is not None
        assert mix_response.text == response["text"]
        assert mix_response.success == response["success"]
        assert 1.12 == response["dec"]
        assert str(mix_response.uid) == response["uid"]
        assert time.isoformat() == str(response["dt"])
        assert datetime.datetime.date(time).isoformat() == str(response["date"])
        assert item.title == response["item"]["title"]
        assert len(items) == len(response["items"])



import inspect

from pydiator_core.interfaces import BaseRequest, BasePipeline
from pydiator_core.mediatr_container import BaseMediatrContainer


class DefaultPipeline(BasePipeline):
    def __init__(self, mediatr_container: BaseMediatrContainer):
        self.mediatr_container = mediatr_container

    async def handle(self, req: BaseRequest, **kwargs) -> object:
        req_type_name = req.get_class_name()
        handler = self.mediatr_container.get_requests().get(req_type_name, None)
        if handler is None:
            raise Exception(f"handler_not_found_for_request_:{req_type_name}")

        handle_func = getattr(handler, "handle", None)
        if not callable(handle_func):
            raise Exception("handle_function_has_not_found_in_handler")

        is_async = inspect.iscoroutinefunction(handler.handle)
        if not is_async:
            return handler.handle(req)

        return await handler.handle(req)



from typing import List
from unittest import mock

from pydiator_core.default_pipeline import DefaultPipeline
from pydiator_core.interfaces import BaseResponse, BaseRequest, BaseHandler
from pydiator_core.mediatr_container import MediatrContainer
from tests.base_test_case import BaseTestCase, TestRequest, TestHandler, TestSyncHandler


class TestDefaultPipeline(BaseTestCase):
    def setUp(self):
        pass

    def test_handle_return_exception_when_handler_is_not(self):
        # Given
        pipeline = DefaultPipeline(MediatrContainer())

        # When
        with self.assertRaises(Exception) as context:
            self.async_loop(pipeline.handle(req=TestRequest()))

        # Then
        assert context.exception.args[0] == f'handler_not_found_for_request_:{type(TestRequest()).__name__}'

    def test_handle_return_exception_when_handler_is_not_callable(self):
        # Given
        mock_container = mock.MagicMock()
        mock_container.get_requests.return_value.get.return_value = {}
        self.pipeline = DefaultPipeline(mock_container)

        # When
        with self.assertRaises(Exception) as context:
            self.async_loop(self.pipeline.handle(req=TestRequest()))

        # Then
        assert 'handle_function_has_not_found_in_handler' == context.exception.args[0]

    def test_handle_return_handle_response(self):
        # Given
        container = MediatrContainer()
        container.register_request(TestRequest, TestHandler())
        self.pipeline = DefaultPipeline(container)

        # When
        response = self.async_loop(self.pipeline.handle(req=TestRequest()))

        # Then
        assert isinstance(response, BaseResponse)

    def test_handle_return_handle_response_when_handle_is_sync(self):
        # Given
        container = MediatrContainer()
        container.register_request(TestRequest, TestSyncHandler())
        self.pipeline = DefaultPipeline(container)

        # When
        response = self.async_loop(self.pipeline.handle(req=TestRequest()))

        # Then
        assert isinstance(response, BaseResponse)

    def test_handle_return_handle_response_with_list_when_handle_is_sync(self):
        # Given
        class TestRequest(BaseRequest):
            def __init__(self):
                return

        class Item:
            def __init__(self, id: int, title: str):
                self.id: int = id,
                self.title = title

        class TestResponse(BaseResponse):
            def __init__(self, items: List[Item]):
                self.items = items

        class TestSyncHandler(BaseHandler):
            def handle(self, req: BaseRequest):
                return TestResponse(items=[Item(id=1, title="item1"), Item(id=2, title="item2")])

        container = MediatrContainer()
        container.register_request(TestRequest, TestSyncHandler())
        self.pipeline = DefaultPipeline(container)

        # When
        response = self.async_loop(self.pipeline.handle(req=TestRequest()))

        # Then
        assert isinstance(response, BaseResponse)
        assert 2 == len(response.items)



from typing import List
from pydiator_core.interfaces import BaseRequest, BaseHandler, BasePipeline, BaseNotification, BaseNotificationHandler, \
    BaseMediatrContainer


class MediatrContainer(BaseMediatrContainer):

    def __init__(self):
        self.__requests = {}
        self.__notifications = {}
        self.__pipelines = []
        self.__base_request_get_class_method_name = BaseRequest.get_class_name.__name__
        self.__base_notification_get_class_method_name = BaseNotification.get_class_name.__name__

    def register_request(self, req: type, handler: BaseHandler):
        if not isinstance(handler, BaseHandler):
            return

        if hasattr(req, self.__base_request_get_class_method_name) and callable(
                getattr(req, self.__base_request_get_class_method_name)):
            req_type = getattr(req, self.__base_request_get_class_method_name)()
            self.__requests[req_type] = handler

    def register_pipeline(self, pipeline: BasePipeline):
        self.__pipelines.append(pipeline)

    def register_notification(self, notification: type, handlers: List[BaseNotificationHandler]):
        if hasattr(notification, self.__base_notification_get_class_method_name) and callable(
                getattr(notification, self.__base_notification_get_class_method_name)):
            notification_type = getattr(notification, self.__base_notification_get_class_method_name)()
            self.__notifications[notification_type] = handlers

    def get_requests(self):
        return self.__requests

    def get_notifications(self):
        return self.__notifications

    def get_pipelines(self):
        return self.__pipelines

    def prepare_pipes(self, pipeline: BasePipeline):
        self.register_pipeline(pipeline)
        pipelines_length = len(self.__pipelines)
        if pipelines_length == 1:
            return

        for i in range(pipelines_length - 1, -1, -1):
            if 0 == i:
                break
            self.__pipelines[i - 1].set_next(self.__pipelines[i])



import enum

from abc import ABC, abstractmethod
from typing import List
from pydiator_core.serializer import JsonSerializable


class BaseRequest:
    def __init__(self):
        pass

    @classmethod
    def get_class_name(cls):
        return cls.__name__


class BaseResponse(JsonSerializable):
    def __init__(self):
        pass


class BaseHandler(ABC):
    def __init__(self):
        pass

    @abstractmethod
    async def handle(self, req: BaseRequest):
        pass


class BaseNotification:
    def __init__(self):
        pass

    @classmethod
    def get_class_name(cls):
        return cls.__name__


class BaseNotificationHandler(ABC):
    def __init__(self):
        pass

    @abstractmethod
    async def handle(self, notification: BaseNotification):
        pass


class CacheType(enum.Enum):
    NONE = 0
    DISTRIBUTED = 1


class BaseCacheable(ABC):

    def __init__(self):
        self._no_cache = False

    @abstractmethod
    def get_cache_key(self) -> str:
        pass

    @abstractmethod
    def get_cache_duration(self) -> int:
        pass

    @abstractmethod
    def get_cache_type(self) -> CacheType:
        pass

    def set_no_cache(self):
        self._no_cache = True

    def is_no_cache(self):
        if hasattr(self, '_no_cache'):
            return self._no_cache
        return False


class BasePipeline(ABC):
    _next = None

    def __init__(self):
        pass

    def next(self) -> object:
        return self._next

    def set_next(self, handler=None):
        self._next = handler

    def has_next(self):
        return self._next is not None

    @abstractmethod
    async def handle(self, req: BaseRequest, **kwargs) -> object:
        pass


class BaseMediatr(ABC):

    @abstractmethod
    async def send(self, req: BaseRequest) -> object:
        pass

    @abstractmethod
    async def publish(self, notification: BaseNotification, throw_exception: bool = False):
        pass


class BaseMediatrContainer(ABC):

    @abstractmethod
    def register_request(self, req: BaseRequest, handler: BaseHandler):
        pass

    @abstractmethod
    def register_pipeline(self, pipeline: BasePipeline):
        pass

    @abstractmethod
    def register_notification(self, notification: BaseNotification, handlers: List[BaseNotificationHandler]):
        pass

    @abstractmethod
    def get_requests(self):
        pass

    @abstractmethod
    def get_notifications(self):
        pass

    @abstractmethod
    def get_pipelines(self):
        pass

    @abstractmethod
    def prepare_pipes(self, pipeline: BasePipeline):
        pass


class BaseCacheProvider(ABC):
    def __init__(self):
        pass

    @abstractmethod
    def add(self, key: str, value, expires):
        pass

    @abstractmethod
    def get(self, key: str):
        pass

    @abstractmethod
    def exist(self, key: str):
        pass

    @abstractmethod
    def delete(self, key: str):
        pass

    @abstractmethod
    def check_connection(self):
        pass



from pydiator_core.mediatr_container import MediatrContainer
from pydiator_core.interfaces import BaseRequest, BaseNotification
from tests.base_test_case import BaseTestCase, TestPipeline, TestRequest, TestResponse, TestHandler, TestNotification, \
    TestNotificationHandler


class TestMediatrContainer(BaseTestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_read_default_values_when_create_instance(self):
        # Given

        # When
        container = MediatrContainer()

        # Then
        assert container.get_requests() == {}
        assert container.get_notifications() == {}
        assert container.get_pipelines() == []
        assert container._MediatrContainer__base_notification_get_class_method_name == BaseNotification.get_class_name.__name__
        assert container._MediatrContainer__base_notification_get_class_method_name == BaseRequest.get_class_name.__name__

    def test_register_pipeline(self):
        # Given
        container = MediatrContainer()
        response = TestResponse(success=True)

        # When
        container.register_pipeline(TestPipeline(response))

        # Then
        assert container.get_requests() == {}
        assert container.get_notifications() == {}
        assert len(container.get_pipelines()) == 1

    def test_get_pipelines(self):
        # Given
        response = TestResponse(success=True)
        container = MediatrContainer()
        pipeline = TestPipeline(response)

        # When
        container.register_pipeline(pipeline)
        response = container.get_pipelines()

        # Then
        assert container.get_requests() == {}
        assert container.get_notifications() == {}
        assert len(response) == 1
        assert response[0] is pipeline

    def test_register_notification_when_added_notification(self):
        # Given
        container = MediatrContainer()

        # When
        container.register_notification(TestNotification, [TestNotificationHandler()])

        # Then
        assert container.get_requests() == {}
        assert container.get_pipelines() == []
        assert len(container.get_notifications()) == 1

    def test_register_notification_when_notification__is_not_instance_of_base_notification(self):
        # Given
        container = MediatrContainer()

        # When
        container.register_notification(MediatrContainer, [TestNotificationHandler()])
        response = container.get_notifications()

        # Then
        assert container.get_requests() == {}
        assert container.get_pipelines() == []
        assert len(response) == 0

    def test_get_notifications(self):
        # Given
        container = MediatrContainer()
        handlers = [TestNotificationHandler()]

        # When
        container.register_notification(TestNotification, handlers)
        response = container.get_notifications()

        # Then
        assert container.get_requests() == {}
        assert container.get_pipelines() == []
        assert len(response) == 1
        assert response[TestNotification.get_class_name()] is not None
        assert response[TestNotification.get_class_name()] == handlers

    def test_register_request(self):
        # Given
        request = TestRequest()
        handler = TestHandler()
        container = MediatrContainer()

        # When
        container.register_request(req=TestRequest, handler=handler)

        # Then
        assert container.get_notifications() == {}
        assert len(container.get_requests()) == 1
        assert container.get_requests()[TestRequest.get_class_name()] is not None
        assert container.get_requests()[TestRequest.get_class_name()] == handler

    def test_register_request_return_when_request_is_not_instance_of_base_request(self):
        # Given
        handler = TestHandler()
        container = MediatrContainer()

        # When
        container.register_request(req=TestResponse, handler=handler)

        # Then
        assert len(container.get_requests()) == 0

    def test_register_request_return_when_handler_is_not_instance_of_base_handler(self):
        # Given
        container = MediatrContainer()

        # When
        container.register_request(req=TestRequest, handler={})

        # Then
        assert len(container.get_requests()) == 0

    def test_prepare_pipes_when_pipelines_length_is_equal_1(self):
        # Given
        response = TestResponse(success=True)
        container = MediatrContainer()
        test_pipeline = TestPipeline(response)

        # When
        container.prepare_pipes(test_pipeline)

        # Then
        assert len(container.get_pipelines()) == 1
        assert test_pipeline.has_next() is False

    def test_prepare_pipes_when_pipelines_length_is_greater_than_1(self):
        # Given
        response = TestResponse(success=True)
        test_pipeline = TestPipeline(response)
        test_default_pipeline = TestPipeline(response)

        container = MediatrContainer()
        container.register_pipeline(test_pipeline)

        # When
        container.prepare_pipes(test_default_pipeline)

        # Then
        assert len(container.get_pipelines()) == 2
        assert test_pipeline.has_next()
        assert test_default_pipeline.has_next() is False
